#include "hashio.h"
#include <stdio.h>
#include <string.h>
#include "personio.h"
#include "person.h"

typedef unsigned long  ul_t;


void phash_fn_add(phash_table_t *phtbl, char *fn)
{
    FILE      *f = fopen(fn, "r");
    char      line[0x100]; /* == 256 */

    if (!f)
    {
        fprintf(stderr, "Failed to open %s\n", fn);
    }

    while (fgets(line, sizeof(line), f))
    {
        if (line[0] != '#')
        {
            person_t *p = person_malloc();
            if (line2person(p, line))
            {
                phash_insert(phtbl, p);
            }
        }
    }

    if (f) { fclose(f); }
}


void phash_fn_delete(phash_table_t *phtbl, char *fn)
{
    FILE      *f = fopen(fn, "r");
    char      line[0x100]; /* == 256 */

    if (!f)
    {
        fprintf(stderr, "Failed to open %s\n", fn);
    }

    while (fgets(line, sizeof(line), f))
    {
        if (line[0] != '#')
        {
            int deleted;
            /* line has a name, followed by colon are just a new-line  */
            char *end_of_name = strchr(line, ':');

            if (end_of_name == NULL)
            {
                end_of_name = strchr(line, '\n');
            }
            if (end_of_name)
            {
                *end_of_name = '\0'; /* trim */
            }
            deleted = phash_delete(phtbl, line);
            if (!deleted)
            {
                fprintf(stderr, "Not found in hash: name = '%s'\n", line);
            }
        }
    }

    if (f) { fclose(f); }
}


void  phash_print(phash_table_t *phtbl, char *fn, char *title)
{
    phash_print_segment(phtbl, fn, 0, phtbl->hash_size, title);
}


void  phash_print_segment(
    phash_table_t *phtbl,
    char          *fn,
    ul_t          b,
    ul_t          e, 
    char          *title)
{
    int   use_stdout = (strcmp(fn, "-") == 0);
    FILE  *f = (use_stdout ? stdout : fopen(fn, "a"));
    ul_t  hi;
    
    fprintf(f, "%s\n{ #=%ld\n", title, phtbl->n_persons);
    for (hi = b;  hi < e; hi++)
    {
        person_node_t  *pnode;

        fprintf(f, "  [%ld]:\n", hi);
        for (pnode = phtbl->entries[hi];  pnode;  pnode = pnode->next)
        {
            char line[0x100]; /* large enough */
            person2line(line, pnode->person);
            fprintf(f, "    %s\n", line);
        }
    }
    fprintf(f, "}\n");
    
    if (!use_stdout)
    {
        fclose(f);
    }
}



