/* Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2011/March/26 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "person.h"
#include "personio.h"
#include "phash.h"
#include "hashio.h"



typedef unsigned long  ul_t;

typedef struct
{
    char  *fnadd;
    char  *fndel;
    ul_t  hi_begin;
    ul_t  hi_end;
    int   show_segment;
} prog_params_t;


static void usage(char *progname)
{
    fprintf(stderr, 
    "Usage:\n"
    "  %s\n"
    "    <-fnadd <filename>  # File with persons data to add from\n"
    "    [-fndel <filename>] # Optional File with persons names to delete\n"
    "    [-hseg  <n> <n>]    # Optional [)-segment hash-indices to print\n"
    , progname);
}


/* Return 0 if ok, 1 otherwise */
static int get_params(prog_params_t *pp, int argc, char **argv)
{
    int  ai;
    char *error = NULL;

    /* Defaults */
    pp->fnadd        = NULL;
    pp->fndel        = NULL;
    pp->hi_begin     = 0;
    pp->hi_end       = 0;
    pp->show_segment = 0;

    for (ai = 1; (error == NULL) && (ai < argc); ai++)
    {
        char *option = argv[ai];
        char *value = (ai + 1 < argc ? argv[ai + 1] : NULL);

        if (strcmp(option, "-fnadd") == 0)
        {
            pp->fnadd = value;  ++ai;
        }
        else if (strcmp(option, "-fndel") == 0)
        {
            pp->fndel = value;  ++ai;
        }
        else if (strcmp(option, "-hseg") == 0)
        {
            pp->show_segment = 1;
            pp->hi_begin = (++ai < argc ? strtoul(argv[ai], 0, 0) : 1);
            pp->hi_end = (++ai < argc ? strtoul(argv[ai], 0, 0) : 0);
        }
    }

    if (pp->fnadd == NULL)
    {
        error = "Missing -fnadd option";
    }
    else if (pp->show_segment)
    {
        if (pp->hi_begin > pp->hi_end)
        {
            error = "Bad segment values";
        }
    }
    if (error) 
    {
        fprintf(stderr, "Error: %s\n", error);
        usage(argv[0]);
    }

    return (error ? 1 : 0);
}


static void htbl_pr_allorseg(
    phash_table_t *phtbl,
    prog_params_t *pp,
    char          *title)
{
    if (pp->show_segment)
    {
        phash_print_segment(phtbl, "-", pp->hi_begin, pp->hi_end, title);
    }
    else
    {
        phash_print(phtbl, "-", title);
    }
}


int main(int argc, char **argv)
{
    prog_params_t  params;
    int            rc = get_params(&params, argc, argv);
    
    if (rc == 0)
    {
        phash_table_t  phash_table;

        phash_init(&phash_table);
        phash_print(&phash_table, "-", "After initialized");

        phash_fn_add(&phash_table, params.fnadd);
        htbl_pr_allorseg(&phash_table, &params, "After Additions");

        if (params.fndel)
        {
            phash_fn_delete(&phash_table, params.fndel);
            htbl_pr_allorseg(&phash_table, &params, "After Deletions");
        }

        phash_clear(&phash_table);
        phash_print(&phash_table, "-", "After clear");
    }
    return rc;
}
