/* Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2011/March/26 */
#if !defined(phash_H)
#define phash_H

#include "person.h"

/* Persons doubly linked list  hash table */
typedef struct _person_node
{
    person_t             *person;
    struct _person_node  *prev;
    struct _person_node  *next;
} person_node_t;


/* Persons  hash table */
typedef struct
{
    person_node_t  **entries;    /* Array of list (of prime size */
    unsigned long  hash_size; /* Size of entry array */
    unsigned long  n_persons; /* Total number of persons in table */
} phash_table_t;


/* initilazed the table as empty */
extern void phash_init(phash_table_t *phtbl);

/* Insert person to table. Return 1 if inserted, 0 otherwise 
 * After insertion, p is owned by the table!
 */
extern int phash_insert(phash_table_t *phtbl, person_t *p);

/* Find person in table given her name. 
 * Return pointer to person, NULL if not found.
 */
extern person_t  *phash_find(phash_table_t *phtbl, char *name);

/* Find and delete person in table givem her name.
 * Return 1 if deleted, 0 othereise.
 */
extern int phash_delete(phash_table_t *phtbl, char *name);

/* Delete all entries, and clear the table.
 * Note: phtbl pointer itself is not freed.
 */
extern void phash_clear(phash_table_t *phtbl);

#endif /* phash_H */
